@@ -422,6 +422,8 @@ struct componentOps formOps = {
     newtDefaultMappedHandler,
 } ;
 
+int needResize = 0;
+
 static inline int componentFits(newtComponent co, int compNum) {
     struct form * form = co->data;
     struct element * el = form->elements + compNum;
@@ -448,6 +450,7 @@ newtComponent newtForm(newtComponent vertBar, void * help, int flags) {
 
     co->takesFocus = 0;			/* we may have 0 components */
     co->ops = &formOps;
+    co->destroyCallback = NULL;
 
     form->help = help;
     form->flags = flags;
@@ -471,7 +474,6 @@ newtComponent newtForm(newtComponent vertBar, void * help, int flags) {
     if (!(form->flags & NEWT_FLAG_NOF12)) {
 	newtFormAddHotKey(co, NEWT_KEY_F12);
     }
-    /* TEST */ newtFormAddHotKey (co, NEWT_KEY_ESCAPE);
 
     if (vertBar)
 	form->vertBar = vertBar;
@@ -579,10 +581,9 @@ static void formPlace(newtComponent co, int left, int top) {
     co->left = left;
 
     for (i = 0, el = form->elements; i < form->numComps; i++, el++) {
-	el->co->top += vertDelta;
 	el->top += vertDelta;
-	el->co->left += horizDelta;
 	el->left += horizDelta;
+	el->co->ops->place(el->co, el->co->left, el->co->top);
     }
 }
 
@@ -604,7 +605,7 @@ void newtDrawForm(newtComponent co) {
 	} else {
 	    /* only draw it if it'll fit on the screen vertically */
 	    if (componentFits(co, i)) {
-		el->co->top = el->top - form->vertOffset;
+		el->co->ops->place(el->co, el->left, el->top - form->vertOffset);
 		el->co->ops->mapped(el->co, 1);
 		el->co->ops->draw(el->co);
 	    } else {
@@ -742,8 +743,9 @@ static struct eventResult formEvent(newtComponent co, struct event ev) {
 	    else if (new >= form->numComps)
 		new = (form->numComps - 1);
 
-	    while (!form->elements[new].co->takesFocus)
-		new = new - dir;
+	    while (!form->elements[new].co->takesFocus &&
+		    new - dir >= 0 && new - dir < form->numComps)
+		new -= dir;
 	} else {
 	    do {
 		new += dir;
@@ -753,6 +755,9 @@ static struct eventResult formEvent(newtComponent co, struct event ev) {
 			new = form->numComps - 1;
 		    else if (new >= form->numComps)
 			new = 0;
+		    if (new == form->currComp)
+			/* back where we started */
+			return er;
 		} else if (new < 0 || new >= form->numComps)
 		    return er;
 	    } while (!form->elements[new].co->takesFocus);
@@ -786,6 +791,26 @@ static struct eventResult formEvent(newtComponent co, struct event ev) {
     return er;
 }
 
+/* Destroy a component.  Components which have been added to a form
+ * are destroyed when the form is destroyed; this is just for the
+ * (rare) case of components which for whatever reason weren't added
+ * to a form.
+ */
+void newtComponentDestroy(newtComponent co) {
+    /* If the user registered a destroy callback for this component,
+     * now is a good time to call it.
+     */
+    if (co->destroyCallback)
+        co->destroyCallback(co, co->destroyCallbackData);
+
+    if (co->ops->destroy) {
+        co->ops->destroy(co);
+    } else {
+        if (co->data) free(co->data);
+	free(co);
+    }
+}
+
 /* this also destroys all of the components on the form */
 void newtFormDestroy(newtComponent co) {
     newtComponent subco;
@@ -795,12 +820,7 @@ void newtFormDestroy(newtComponent co) {
     /* first, destroy all of the components */
     for (i = 0; i < form->numComps; i++) {
 	subco = form->elements[i].co;
-	if (subco->ops->destroy) {
-	    subco->ops->destroy(subco);
-	} else {
-	    if (subco->data) free(subco->data);
-	    free(subco);
-	}
+	newtComponentDestroy(subco);
     }
 
     if (form->hotKeys) free(form->hotKeys);
@@ -848,11 +868,19 @@ void newtFormSetSize(newtComponent co) {
     co->width = 0;
     if (!form->fixedHeight) co->height = 0;
 
-    co->top = form->elements[0].co->top;
-    co->left = form->elements[0].co->left;
+    co->top = -1;
+    co->left = -1;
+
     for (i = 0, el = form->elements; i < form->numComps; i++, el++) {
 	if (el->co->ops == &formOps)
 	    newtFormSetSize(el->co);
+	else if (el->co == form->vertBar)
+	    continue;
+
+	if (co->top == -1) {
+	    co->top = el->co->top;
+	    co->left = el->co->left;
+	}
 
  	el->left = el->co->left;
  	el->top = el->co->top;
@@ -866,6 +894,7 @@ void newtFormSetSize(newtComponent co) {
 	if (co->top > el->co->top) {
 	    delta = co->top - el->co->top;
 	    co->top -= delta;
+	    form->numRows += delta;
 	    if (!form->fixedHeight)
 		co->height += delta;
 	}
@@ -911,7 +940,8 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
     newtDrawForm(co);
 
     if (form->currComp == -1) {
-	gotoComponent(form, 0);
+	if (form->numComps)
+	    gotoComponent(form, 0);
     } else
 	gotoComponent(form, form->currComp);
 
@@ -943,16 +973,19 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 	if (form->timer) {
 	    /* Calculate when we next need to return with a timeout. Do
 	       this inside the loop in case a callback resets the timer. */
-	    if (!form->lastTimeout.tv_sec && !form->lastTimeout.tv_usec)
-		gettimeofday(&form->lastTimeout, NULL);
+	    gettimeofday(&now, 0);
+
+	    if ((!form->lastTimeout.tv_sec && !form->lastTimeout.tv_usec) ||
+		    now.tv_sec < form->lastTimeout.tv_sec ||
+		    (now.tv_sec == form->lastTimeout.tv_sec &&
+		     now.tv_usec < form->lastTimeout.tv_usec))
+		form->lastTimeout = now;
 
 	    nextTimeout.tv_sec = form->lastTimeout.tv_sec + 
 		    (form->timer / 1000);
 	    nextTimeout.tv_usec = form->lastTimeout.tv_usec + 
 				    (form->timer % 1000) * 1000;
 
-	    gettimeofday(&now, 0);
-
 	    if (now.tv_sec > nextTimeout.tv_sec) {
 		timeout.tv_sec = timeout.tv_usec = 0;
 	    } else if (now.tv_sec == nextTimeout.tv_sec) {
@@ -974,6 +1007,11 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 	    timeout.tv_sec = timeout.tv_usec = 0;
 	}
 
+	if (needResize) {
+		needResize = 0;
+		newtResizeScreen(1);
+	}
+
 	i = select(max + 1, &readSet, &writeSet, &exceptSet, 
 			form->timer ? &timeout : NULL);
 	if (i < 0) continue;	/* ?? What should we do here? */
@@ -1013,11 +1051,6 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 
 		key = newtGetKey();
 
-		if (key == NEWT_KEY_RESIZE) {
-		    newtResizeScreen(1);
-		    continue;
-		}
-
 		for (i = 0; i < form->numHotKeys; i++) {
 		    if (form->hotKeys[i] == key) {
 			es->reason = NEWT_EXIT_HOTKEY;
@@ -1027,8 +1060,17 @@ void newtFormRun(newtComponent co, struct newtExitStruct * es) {
 		    }
 		}
 
-		if (key == NEWT_KEY_F1 && form->helpTag && form->helpCb)
+		if (key == NEWT_KEY_F1 && form->helpTag && form->helpCb) {
+		    if (form->currComp != -1) {
+			ev.event = EV_UNFOCUS;
+			sendEvent(form->elements[form->currComp].co, ev);
+		    }
 		    form->helpCb(co, form->helpTag);
+		    if (form->currComp != -1) {
+			ev.event = EV_FOCUS;
+			sendEvent(form->elements[form->currComp].co, ev);
+		    }
+		}
 
 		if (!done) {
 		    ev.event = EV_KEYPRESS;
@@ -1108,6 +1150,13 @@ void newtComponentAddCallback(newtComponent co, newtCallback f, void * data) {
     co->callbackData = data;
 }
 
+/* Add a callback which is called when the component is destroyed. */
+void newtComponentAddDestroyCallback(newtComponent co,
+				newtCallback f, void * data) {
+    co->destroyCallback = f;
+    co->destroyCallbackData = data;
+}
+
 void newtComponentTakesFocus(newtComponent co, int val) {
     co->takesFocus = val;
 }
