@@ -356,11 +356,15 @@ static PyObject * screenSize(PyObject * s, PyObject * args) {
 static void helpCallbackMarshall(newtComponent co, void * data) {
     PyObject * args, * result;
 
+    PyGILState_STATE _state = PyGILState_Ensure();
+
     args = Py_BuildValue("(O)", data);
     result = PyEval_CallObject(helpCallback.cb, args);
     Py_DECREF (args);
     Py_XDECREF(result);
 
+    PyGILState_Release(_state);
+
     return;
 }
 
@@ -368,6 +372,8 @@ static void suspendCallbackMarshall(void * data) {
     struct callbackStruct * scs = data;
     PyObject * args, * result;
 
+    PyGILState_STATE _state = PyGILState_Ensure();
+
     if (scs->data) {
 	args = Py_BuildValue("(O)", scs->data);
 	result = PyEval_CallObject(scs->cb, args);
@@ -382,6 +388,8 @@ static void suspendCallbackMarshall(void * data) {
 
     Py_XDECREF(result);
 
+    PyGILState_Release(_state);
+
     return;
 }
 
@@ -389,6 +397,8 @@ static void callbackMarshall(newtComponent co, void * data) {
     struct callbackStruct * scs = data;
     PyObject * args, * result;
 
+    PyGILState_STATE _state = PyGILState_Ensure();
+
     if (scs->data) {
 	args = Py_BuildValue("(O)", scs->data);
 	result = PyEval_CallObject(scs->cb, args);
@@ -403,6 +413,8 @@ static void callbackMarshall(newtComponent co, void * data) {
     
     Py_XDECREF(result);
 
+    PyGILState_Release(_state);
+
     return;
 }
 
@@ -563,7 +575,9 @@ static PyObject * messageWindow(PyObject * s, PyObject * args) {
     if (!PyArg_ParseTuple(args, "ss|s", &title, &text, &okbutton)) 
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     newtWinMessage(title, okbutton, text);
+    Py_END_ALLOW_THREADS
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -579,7 +593,9 @@ static PyObject * choiceWindow(PyObject * s, PyObject * args) {
 			  &cancelbutton)) 
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     rc = newtWinChoice(title, okbutton, cancelbutton, text);
+    Py_END_ALLOW_THREADS
 
     return Py_BuildValue("i", rc);
 }
@@ -592,7 +608,9 @@ static PyObject * ternaryWindow(PyObject * s, PyObject * args) {
 			  &button3)) 
 	return NULL;
 
+    Py_BEGIN_ALLOW_THREADS
     rc = newtWinTernary(title, button1, button2, button3, text);
+    Py_END_ALLOW_THREADS
 
     return Py_BuildValue("i", rc);
 }
@@ -883,7 +901,9 @@ static PyObject * formAdd(snackForm * s, PyObject * args) {
 static PyObject * formRun(snackForm * s, PyObject * args) {
     struct newtExitStruct result;
 
+    Py_BEGIN_ALLOW_THREADS
     newtFormRun(s->fo, &result);
+    Py_END_ALLOW_THREADS
 
     if (result.reason == NEWT_EXIT_HOTKEY)
 	return Py_BuildValue("(si)", "hotkey", result.u.key);
@@ -1287,12 +1307,6 @@ static PyObject * pywstrlen(PyObject * s, PyObject * args)
 void init_snack(void) {
     PyObject * d, * m;
 
-#ifdef ENABLE_NLS
-    setlocale (LC_ALL, "");
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
-#endif
-
     m = Py_InitModule("_snack", snackModuleMethods);
     d = PyModule_GetDict(m);
 
@@ -1313,6 +1327,21 @@ void init_snack(void) {
     PyDict_SetItemString(d, "FORM_EXIT_TIMER", PyString_FromString("timer"));
     PyDict_SetItemString(d, "FORM_EXIT_FDREADY", PyString_FromString("fdready"));
 
+    PyDict_SetItemString(d, "KEY_TAB", PyInt_FromLong(NEWT_KEY_TAB));
+    PyDict_SetItemString(d, "KEY_ENTER", PyInt_FromLong(NEWT_KEY_ENTER));
+    PyDict_SetItemString(d, "KEY_SUSPEND", PyInt_FromLong(NEWT_KEY_SUSPEND));
+    PyDict_SetItemString(d, "KEY_UP", PyInt_FromLong(NEWT_KEY_UP));
+    PyDict_SetItemString(d, "KEY_DOWN", PyInt_FromLong(NEWT_KEY_DOWN));
+    PyDict_SetItemString(d, "KEY_LEFT", PyInt_FromLong(NEWT_KEY_LEFT));
+    PyDict_SetItemString(d, "KEY_RIGHT", PyInt_FromLong(NEWT_KEY_RIGHT));
+    PyDict_SetItemString(d, "KEY_BACKSPACE", PyInt_FromLong(NEWT_KEY_BKSPC));
+    PyDict_SetItemString(d, "KEY_DELETE", PyInt_FromLong(NEWT_KEY_DELETE));
+    PyDict_SetItemString(d, "KEY_HOME", PyInt_FromLong(NEWT_KEY_HOME));
+    PyDict_SetItemString(d, "KEY_END", PyInt_FromLong(NEWT_KEY_END));
+    PyDict_SetItemString(d, "KEY_UNTAB", PyInt_FromLong(NEWT_KEY_UNTAB));
+    PyDict_SetItemString(d, "KEY_PAGEUP", PyInt_FromLong(NEWT_KEY_PGUP));
+    PyDict_SetItemString(d, "KEY_PAGEGDOWN", PyInt_FromLong(NEWT_KEY_PGDN));
+    PyDict_SetItemString(d, "KEY_INSERT", PyInt_FromLong(NEWT_KEY_INSERT));
     PyDict_SetItemString(d, "KEY_F1", PyInt_FromLong(NEWT_KEY_F1));
     PyDict_SetItemString(d, "KEY_F2", PyInt_FromLong(NEWT_KEY_F2));
     PyDict_SetItemString(d, "KEY_F3", PyInt_FromLong(NEWT_KEY_F3));
